//******************************************************************************
#include "PrecompiledHeader.h"

#include "RenamerApp.h"
#include "Log.h"
#include "RenamerThread.h"
//******************************************************************************
using namespace MP3Renamer;
//******************************************************************************
IMPLEMENT_APP( RenamerApp)

BEGIN_EVENT_TABLE( RenamerApp, wxApp)
	EVT_BUTTON(		OK,			RenamerApp::_onOK)
	EVT_BUTTON(		Help,		RenamerApp::_onHelp)
	EVT_BUTTON(		Cancel,		RenamerApp::_onCancel)
	EVT_BUTTON(		Folder,		RenamerApp::_onFolderButton)
	EVT_TEXT_ENTER( RootFolder,	RenamerApp::_onFolderChange)
	EVT_TEXT_ENTER( Mask,		RenamerApp::_onMaskChange)
END_EVENT_TABLE()
//******************************************************************************
int g_changedFiles = 0;
int g_nbFilesToChange = 0;
wxProgressDialog * g_dialog = NULL;
wxString g_path;
wxString g_mask;
//******************************************************************************

void CleanupFolder( wxDir * p_folder)
{
	if ( ! p_folder->HasFiles())
	{
		wxString l_name = p_folder->GetName();
		delete p_folder;
		if ( ! wxFileName::Rmdir( l_name))
		{
			Log::LogMessage( "Can't remove folder " + l_name);
		}
	}
	else
	{
		delete p_folder;
	}
}
//******************************************************************************

MP3RFolder :: MP3RFolder( const wxString & p_name)
	:	m_name	( p_name),
		m_nbMP3s( 0),
		m_folder( new wxDir( p_name))
{
}

//******************************************************************************

MP3RFolder :: ~MP3RFolder()
{
	m_nbMP3s = 0;
	for (size_t i = 0 ; i < m_childs.size() ; i++)
	{
		delete m_childs[i];
	}
	m_childs.clear();
	m_files.clear();
}

//******************************************************************************

void MP3RFolder :: ChangeFiles()
{
	if (m_childs.size() > 0)
	{
		for (size_t i = 0 ; i < m_childs.size() ; i++)
		{
			m_childs[i]->ChangeFiles();
		}
	}

	if (m_files.size() > 0)
	{
		RenamerThread * l_thread;
		std::vector <int> l_threadIDs;
		for (size_t i = 0 ; i < m_files.size() ; i++)
		{
			l_thread = new RenamerThread( g_path, m_files[i], g_mask);
			l_thread->Create();
			l_thread->Run();
			if (l_thread->Wait() != 0)
			{
				Log::LogMessage( "Trouble with " + m_files[i]);
			}
			g_changedFiles++;
			m_nbMP3s--;
			if (m_nbMP3s <= 0)
			{
				CleanupFolder( m_folder);
			}
			g_dialog->Update( g_changedFiles);
		}
	}
	else
	{
		CleanupFolder( m_folder);
	}
}
		
//******************************************************************************

bool RenamerApp :: OnInit()
{
	m_maskTxt = "%gender%\\%artist%\\%album%\\%number% - %title%%ext%";
	m_path = "Dossier";

	m_mainFrame = new wxFrame( NULL, wxID_ANY, "MP3 Renamer", wxPoint( 400, 400), 
							   wxSize( 280, 150), 
							   wxCAPTION | wxSYSTEM_MENU | wxCLOSE_BOX);
	m_mainFrame->Show( true);

	wxPanel * l_container = new wxPanel( m_mainFrame, wxID_ANY, wxPoint( 0, 0), 
										 m_mainFrame->GetClientSize());

	wxPoint l_position = wxPoint( 10, 20);
	wxStaticText * l_txt = new wxStaticText( l_container, wxID_ANY, 
											 "Dossier racine :", l_position);
	wxStaticText * l_txt2 = new wxStaticText( l_container, wxID_ANY, "Masque :", 
											  l_position += wxPoint( 0, 30));

	m_rootFolder = new wxTextCtrl( l_container, RootFolder, "Dossier", 
								   l_position += wxPoint( 80, -30), 
								   wxSize( 100, 20), wxTE_PROCESS_ENTER);
	m_mask = new wxTextCtrl( l_container, Mask, m_maskTxt, 
							 l_position += wxPoint( -20, 30),
							 wxSize( 195, 20), wxTE_PROCESS_ENTER);
	m_folderButton = new wxButton( l_container, Folder, "Parcourir...", 
								   wxPoint( 195, 17), wxSize( 70, 25));

	m_help = new wxButton( l_container, Help, "Aide", wxPoint( 20, 80), wxSize( 60, 25));
	m_ok = new wxButton( l_container, OK, "OK", wxPoint( 105, 80), wxSize( 60, 25));
	m_cancel = new wxButton( l_container, Cancel, "Annuler", wxPoint( 190, 80),
							 wxSize( 60, 25));

	SetTopWindow( m_mainFrame);
	wxIcon l_icon( L"icon.ico");
	m_mainFrame->SetIcon( l_icon);

	m_tree = NULL;

	return true;
}

//******************************************************************************

void RenamerApp :: _onHelp( wxCommandEvent & p_event)
{
	_buildHelpText();
}

//******************************************************************************

void RenamerApp :: _onOK( wxCommandEvent & p_event)
{
	if (wxDir::Exists( m_path))
	{
		g_path = m_path;
		g_mask = m_maskTxt;
		_processChange();
		delete m_log;
		m_mainFrame->Close();
	}
	else
	{
		wxMessageDialog * l_dialog = new wxMessageDialog( m_mainFrame, "Le dossier slectionn n'existe pas", "Erreur", wxID_OK);
		l_dialog->ShowModal();
	}
}

//******************************************************************************

void RenamerApp :: _onCancel( wxCommandEvent & p_event)
{
	m_mainFrame->Close();
}

//******************************************************************************

void RenamerApp :: _onFolderButton( wxCommandEvent & p_event)
{
	wxDirDialog l_dialog( NULL, "Slectionnez un dossier");
	if (l_dialog.ShowModal() == wxID_OK)
	{
		m_path = l_dialog.GetPath();
		m_rootFolder->Clear();
		m_rootFolder->AppendText( m_path);
	}
}

//******************************************************************************

void RenamerApp :: _onFolderChange( wxCommandEvent & p_event)
{
	m_path = m_rootFolder->GetValue();
}

//******************************************************************************

void RenamerApp :: _onMaskChange( wxCommandEvent & p_event)
{
	m_maskTxt = m_mask->GetValue();
}

//******************************************************************************

void RenamerApp :: _buildHelpText()
{
	m_helpFrame = new wxFrame( m_mainFrame, wxID_ANY, "Aide", wxPoint( 300, 300), 
							   wxSize( 400, 400),
							   wxCAPTION | wxSYSTEM_MENU | wxCLOSE_BOX);
	wxTextCtrl * l_helpText = new wxTextCtrl( m_helpFrame, wxID_ANY, wxEmptyString, 
											  wxPoint( 0, 0), m_helpFrame->GetClientSize(),
											  wxTE_MULTILINE | wxTE_READONLY | wxTE_WORDWRAP);
	l_helpText->AppendText( "MP3Renamer v1.0 par DragonJoker\n\n\n");
	l_helpText->AppendText( "Ce renommeur de fichiers mp3 le fait en fonction des informations contenues dans les Tags ID3 de chaque fichier.\n");
	l_helpText->AppendText( "Pour cela, vous pouvez utiliser le masque, qui permet de changer la faon dont les fichiers sont renomms et replacs.\n\n");
	l_helpText->AppendText( "Les directives acceptes dans le masque sont : \n");
	l_helpText->AppendText( "%artist% : l'artiste\n");
	l_helpText->AppendText( "%album% : l'album\n");
	l_helpText->AppendText( "%number% : le numro de piste\n");
	l_helpText->AppendText( "%title% : le titre du morceau\n");
	l_helpText->AppendText( "%ext% : l'extension (.mp3 pour l'instant)\n");
	l_helpText->AppendText( "%gender% : le type de musique\n");
	l_helpText->AppendText( "%year% : l'anne\n");
	l_helpText->AppendText( "\\ ou / : pour sparer les diffrents dossiers\n\n");
	l_helpText->AppendText( "Ainsi, le masque original\n%gender%\\%artist%\\%album%\\%number% - %title%%ext%\npourra donner :\nRock\\Noir Dsir\\Des Visages des Figures\\01 - L'enfant roi.mp3\n\n");
	l_helpText->AppendText( "Le dossier d'origine est celui slectionn pour la recherche de fichiers");
	m_helpFrame->Show();
}

//******************************************************************************

void RenamerApp :: _processChange()
{
	if (m_tree != NULL)
	{
		delete m_tree;
		m_tree = NULL;
	}
	_listFilesToChange();
	_changeFiles();
	Log::LogMessage( "********************************************************************************");
}

//******************************************************************************

void RenamerApp :: _listFilesToChange()
{
	if (m_path.size() == 3)
	{
		m_path = m_path.SubString( 0 , 1);
	}
	m_tree = new MP3RFolder( m_path);
	m_log = new Log( m_tree->m_name + "\\MP3Renamer.log");
	_findFiles( m_tree);
}

//******************************************************************************

void RenamerApp :: _findFiles( MP3RFolder * p_folder)
{
	try
	{
		bool l_searchResult = true;
		WIN32_FIND_DATA l_file;
		wxString l_foldername = p_folder->m_name + "\\*.*";
		HANDLE l_searchHandle = FindFirstFile( l_foldername.c_str(), & l_file);

		wxString l_previousFolder;
		wxString l_previousFile;

		if (l_searchHandle == INVALID_HANDLE_VALUE)
		{
			return;
		}

		wxString l_filename, tmp, l_extension;
		MP3RFolder * l_folder;
		wxFileName l_fileName;
		while (l_searchResult)
		{
			l_searchResult = (FindNextFile( l_searchHandle, & l_file) == TRUE);
			l_fileName = wxFileName( l_file.cFileName);
			l_extension = l_fileName.GetExt().Lower();
			if (l_file.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
			{
				l_filename = l_file.cFileName;
				if (l_filename != ".." && l_filename != "." && l_filename != l_previousFolder)
				{
					l_folder = new MP3RFolder( p_folder->m_name + "\\" + l_file.cFileName);
					p_folder->m_childs.push_back( l_folder);
					l_previousFolder = l_filename;
					_findFiles( l_folder);
				}
			}
			else 
			{
				if (l_extension == ".mp3")
				{
					l_filename = p_folder->m_name + "\\" + l_file.cFileName;
					if (l_previousFile != l_filename)
					{
						Log::LogMessage( l_file.cFileName);
						m_filesToChange.push_back( l_filename.c_str());
						l_previousFile = l_filename;
						p_folder->m_nbMP3s++;
						p_folder->m_files.push_back( l_filename);
						g_nbFilesToChange++;
					}
				}
			}
		}
		FindClose( l_searchHandle);
	}
	catch (...)
	{
		Log::LogMessage( "Erreur");
	}
}

//******************************************************************************

void RenamerApp :: _changeFiles()
{
	g_dialog = new wxProgressDialog( "Progression",  "Progression des changements",
									 static_cast <int>( m_filesToChange.size()),
									 m_mainFrame, wxPD_SMOOTH);
	m_tree->ChangeFiles();
}

//******************************************************************************