//******************************************************************************
#include "PrecompiledHeader.h"

#include "RenamerThread.h"
#include "Log.h"
#include <errno.h>
//******************************************************************************
using namespace MP3Renamer;
//******************************************************************************
static const wxString cs_artist	= "%artist%";
static const wxString cs_album	= "%album%";
static const wxString cs_number	= "%number%";
static const wxString cs_title	= "%title%";
static const wxString cs_ext	= "%ext%";
static const wxString cs_gender	= "%gender%";
static const wxString cs_year	= "%year%";
static const wxString cs_slash	= "\\";
//*****************************************************************************

RenamerThread :: RenamerThread( const wxString & p_path, 
							    const wxString & p_file, 
								const wxString & p_mask)
	:	wxThread( wxTHREAD_JOINABLE),
		m_path		( p_path),
		m_filePath	( p_file),
		m_maskTxt	( p_mask)
{
}
			
//*****************************************************************************

void * RenamerThread :: Entry()
{
	MP3Infos l_infos;
	wxString l_newPath;

	Log::LogMessage( "********************************************************************************");
	_retrieveInfos( l_infos, m_filePath);
	if (_generatePath( l_newPath, l_infos))
	{
//		Log::LogMessage( "OldPath : " + m_filePath + "\nNewPath : " + l_newPath);
		if (rename( m_filePath.c_str(), l_newPath.c_str()) == -1)
		{
			Log::LogMessage( "Couldn't move that file");
		}
	}
	return 0;
}

//******************************************************************************

void RenamerThread :: _normalise( wxString & p_str)
{
	p_str.Replace( "/", "-");
	p_str.Replace( "\\", "-");
	p_str.Replace( "\"", "'");
	p_str.Replace( ":", " ");
	p_str.Replace( "*", " ");
	p_str.Replace( "<", "[");
	p_str.Replace( ">", "]");
	p_str.Replace( "|", "-");
	p_str.Replace( "?", "");
	p_str.Trim();
}

//******************************************************************************

void RenamerThread :: _retrieveInfos( MP3Infos & p_infos, const wxString & p_fileName)
{
	try
	{
		Log::LogMessage( "*	" + p_fileName);

		int l_tmp;
		TagLib::FileRef l_file( p_fileName.c_str());
		TagLib::Tag * l_tag = l_file.tag();

		l_tmp = l_tag->track();
		if (l_tmp < 10)
		{
			p_infos.m_track = "0";
		}
		p_infos.m_track << l_tmp;
		p_infos.m_artist = l_tag->artist().toCString();
		p_infos.m_title = l_tag->title().toCString();
		p_infos.m_album = l_tag->album().toCString();
		p_infos.m_year << l_tag->year();
		p_infos.m_gender = l_tag->genre().toCString();

		_normalise( p_infos.m_artist);
		_normalise( p_infos.m_title);
		_normalise( p_infos.m_album);
		_normalise( p_infos.m_gender);

		Log::LogMessage( "*	Track : " + p_infos.m_track);
		Log::LogMessage( "*	Title : " + p_infos.m_title);
		Log::LogMessage( "*	Album : " + p_infos.m_album);
		Log::LogMessage( "*	Year : " + p_infos.m_year);
		Log::LogMessage( "*	Artist : " + p_infos.m_artist);
		Log::LogMessage( "*	Gender : " + p_infos.m_gender);
	}
	catch ( ... )
	{
		return;
	}
}

//******************************************************************************

bool RenamerThread :: _generatePath( wxString & p_generatedPath, const MP3Infos & p_infos)
{
//	Log::LogMessage( m_maskTxt);
	p_generatedPath = m_maskTxt;
	if (p_generatedPath.Replace( cs_artist, p_infos.m_artist) > 0 && p_infos.m_artist.empty())
	{
		return false;
	}

	if (p_infos.m_album.empty())
	{
		p_generatedPath.Replace( cs_album, "Unknown album");
	}
	else
	{
		p_generatedPath.Replace( cs_album, p_infos.m_album);
	}

	if (p_infos.m_gender.empty())
	{
		p_generatedPath.Replace( cs_gender, "Unknown");
	}
	else
	{
		p_generatedPath.Replace( cs_gender, p_infos.m_gender);
	}

	if (p_generatedPath.Replace( cs_title, p_infos.m_title) > 0 && p_infos.m_title.empty())
	{
		return false;
	}

	if (p_generatedPath.Replace( cs_number, p_infos.m_track) > 0 && p_infos.m_track.empty())
	{
		return false;
	}

	if (p_infos.m_year.empty())
	{
		p_generatedPath.Replace( cs_year, "0");
	}
	else
	{
		p_generatedPath.Replace( cs_year, p_infos.m_year);
	}

	p_generatedPath.Replace( cs_ext, ".mp3");
	if (m_path.size() > 3)
	{
		p_generatedPath = m_path + "\\" + p_generatedPath;
	}
	else
	{
		p_generatedPath = m_path + p_generatedPath;
	}
	Log::LogMessage( "*	" + p_generatedPath);

	_generateFolders( p_generatedPath);

	return true;
}

//******************************************************************************

void RenamerThread :: _generateFolders( const wxString & p_path)
{
	wxString l_path = p_path;
	l_path = l_path.Mid( 0, l_path.find_last_of( "\\"));
	if ( ! wxFileName::Mkdir(l_path, 0777, wxPATH_MKDIR_FULL))
	{
		Log::LogMessage( "Directory " + l_path + " not created");
	}
}

//******************************************************************************

void RenamerThread :: _replaceFile( const wxString & p_oldPath,
								    const wxString & p_newPath)
{
	FILE * l_oldFile, * l_newFile;
	fopen_s( & l_oldFile, p_oldPath.c_str(), "rb");
	fopen_s( & l_newFile, p_newPath.c_str(), "wb");
	fclose( l_oldFile);
	fclose( l_newFile);
}

//******************************************************************************